@@ -11,7 +11,7 @@ import (
 	"github.com/google/uuid"
 	"github.com/jackc/pgx/v4/pgxpool"
 	_ "github.com/jackc/pgx/v4/stdlib"
-	pgdl "github.com/quay/claircore/pkg/distlock/postgres"
+	"github.com/quay/claircore/pkg/ctxlock"
 	"github.com/quay/zlog"
 	"github.com/remind101/migrate"
 	"go.opentelemetry.io/otel/baggage"
@@ -21,7 +21,6 @@ import (
 	"github.com/quay/clair/v4/matcher"
 	"github.com/quay/clair/v4/notifier"
 	namqp "github.com/quay/clair/v4/notifier/amqp"
-	"github.com/quay/clair/v4/notifier/keymanager"
 	"github.com/quay/clair/v4/notifier/migrations"
 	"github.com/quay/clair/v4/notifier/postgres"
 	"github.com/quay/clair/v4/notifier/stomp"
@@ -41,19 +40,13 @@ type Service interface {
 	Notifications(ctx context.Context, id uuid.UUID, page *notifier.Page) ([]notifier.Notification, notifier.Page, error)
 	// Deletes the provided notification id
 	DeleteNotifications(ctx context.Context, id uuid.UUID) error
-	// KeyStore returns the notifier's KeyStore.
-	KeyStore(ctx context.Context) notifier.KeyStore
-	// KeyManager returns the notifier's KeyManager.
-	KeyManager(ctx context.Context) *keymanager.Manager
 }
 
 var _ Service = (*service)(nil)
 
 // service is a local implementation of a notifier service.
 type service struct {
-	store      notifier.Store
-	keystore   notifier.KeyStore
-	keymanager *keymanager.Manager
+	store notifier.Store
 }
 
 func (s *service) Notifications(ctx context.Context, id uuid.UUID, page *notifier.Page) ([]notifier.Notification, notifier.Page, error) {
@@ -64,14 +57,6 @@ func (s *service) DeleteNotifications(ctx context.Context, id uuid.UUID) error {
 	return s.store.SetDeleted(ctx, id)
 }
 
-func (s *service) KeyStore(_ context.Context) notifier.KeyStore {
-	return s.keystore
-}
-
-func (s *service) KeyManager(_ context.Context) *keymanager.Manager {
-	return s.keymanager
-}
-
 // Opts configures the notifier service
 type Opts struct {
 	PollInterval     time.Duration
@@ -97,17 +82,11 @@ func New(ctx context.Context, opts Opts) (*service, error) {
 	)
 
 	// initialize store and dist lock pool
-	store, keystore, lockPool, err := storeInit(ctx, opts)
+	store, lockPool, err := storeInit(ctx, opts)
 	if err != nil {
 		return nil, fmt.Errorf("failed to initialize store and lockpool: %v", err)
 	}
 
-	// kick off key manager
-	kmgr, err := keyManagerInit(ctx, keystore)
-	if err != nil {
-		return nil, fmt.Errorf("failed to initialize key manager: %v", err)
-	}
-
 	// check for test mode
 	if tm := os.Getenv("NOTIFIER_TEST_MODE"); tm != "" {
 		zlog.Warn(ctx).
@@ -128,11 +107,15 @@ func New(ctx context.Context, opts Opts) (*service, error) {
 		Int("count", processors).
 		Msg("initializing processors")
 	for i := 0; i < processors; i++ {
-		// processors only use try locks
-		distLock := pgdl.NewPool(lockPool, 0)
+		// Can't re-use a locker because the Process method unconditionally
+		// spawns background goroutines.
+		l, err := ctxlock.New(ctx, lockPool)
+		if err != nil {
+			return nil, err
+		}
 		p := notifier.NewProcessor(
 			i,
-			distLock,
+			l,
 			opts.Indexer,
 			opts.Matcher,
 			store,
@@ -144,7 +127,7 @@ func New(ctx context.Context, opts Opts) (*service, error) {
 	// kick off configured deliverer type
 	switch {
 	case opts.Webhook != nil:
-		if err := webhookDeliveries(ctx, opts, lockPool, store, kmgr); err != nil {
+		if err := webhookDeliveries(ctx, opts, lockPool, store); err != nil {
 			return nil, err
 		}
 	case opts.AMQP != nil:
@@ -158,9 +141,7 @@ func New(ctx context.Context, opts Opts) (*service, error) {
 	}
 
 	return &service{
-		store:      store,
-		keymanager: kmgr,
-		keystore:   keystore,
+		store: store,
 	}, nil
 }
 
@@ -176,24 +157,24 @@ func testModeInit(ctx context.Context, opts *Opts) error {
 	return nil
 }
 
-func storeInit(ctx context.Context, opts Opts) (*postgres.Store, *postgres.KeyStore, *pgxpool.Pool, error) {
+func storeInit(ctx context.Context, opts Opts) (*postgres.Store, *pgxpool.Pool, error) {
 	ctx = baggage.ContextWithValues(ctx,
 		label.String("component", "notifier/service/storeInit"),
 	)
 
 	cfg, err := pgxpool.ParseConfig(opts.ConnString)
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("failed to parse ConnString: %v", err)
+		return nil, nil, fmt.Errorf("failed to parse ConnString: %v", err)
 	}
 	cfg.MaxConns = 30
 	pool, err := pgxpool.ConnectConfig(ctx, cfg)
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("failed to create ConnPool: %v", err)
+		return nil, nil, fmt.Errorf("failed to create ConnPool: %v", err)
 	}
 
 	db, err := sql.Open("pgx", opts.ConnString)
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("failed to open db: %v", err)
+		return nil, nil, fmt.Errorf("failed to open db: %v", err)
 	}
 	defer db.Close()
 
@@ -204,30 +185,16 @@ func storeInit(ctx context.Context, opts Opts) (*postgres.Store, *postgres.KeySt
 		migrator.Table = migrations.MigrationTable
 		err := migrator.Exec(migrate.Up, migrations.Migrations...)
 		if err != nil {
-			return nil, nil, nil, fmt.Errorf("failed to perform migrations: %w", err)
+			return nil, nil, fmt.Errorf("failed to perform migrations: %w", err)
 		}
 	}
 
 	zlog.Info(ctx).Msg("initializing notifier store")
 	store := postgres.NewStore(pool)
-	keystore := postgres.NewKeyStore(pool)
-	return store, keystore, pool, nil
+	return store, pool, nil
 }
 
-func keyManagerInit(ctx context.Context, keystore notifier.KeyStore) (*keymanager.Manager, error) {
-	ctx = baggage.ContextWithValues(ctx,
-		label.String("component", "notifier/service/keyManagerInit"),
-	)
-
-	zlog.Debug(ctx).Msg("initializing keymanager")
-	mgr, err := keymanager.NewManager(ctx, keystore)
-	if err != nil {
-		return nil, err
-	}
-	return mgr, nil
-}
-
-func webhookDeliveries(ctx context.Context, opts Opts, lockPool *pgxpool.Pool, store notifier.Store, keymanager *keymanager.Manager) error {
+func webhookDeliveries(ctx context.Context, opts Opts, lockPool *pgxpool.Pool, store notifier.Store) error {
 	ctx = baggage.ContextWithValues(ctx,
 		label.String("component", "notifier/service/webhookDeliveries"),
 	)
@@ -242,12 +209,17 @@ func webhookDeliveries(ctx context.Context, opts Opts, lockPool *pgxpool.Pool, s
 
 	ds := make([]*notifier.Delivery, 0, deliveries)
 	for i := 0; i < deliveries; i++ {
-		distLock := pgdl.NewPool(lockPool, 0)
-		wh, err := webhook.New(conf, opts.Client, keymanager)
+		// Can't share a ctxlock because the Deliverer object unconditionally
+		// spawns background goroutines.
+		l, err := ctxlock.New(ctx, lockPool)
+		if err != nil {
+			return fmt.Errorf("failed to create locker: %w", err)
+		}
+		wh, err := webhook.New(conf, opts.Client)
 		if err != nil {
 			return fmt.Errorf("failed to create webhook deliverer: %v", err)
 		}
-		delivery := notifier.NewDelivery(i, wh, opts.DeliveryInterval, store, distLock)
+		delivery := notifier.NewDelivery(i, wh, opts.DeliveryInterval, store, l)
 		ds = append(ds, delivery)
 	}
 	for _, d := range ds {
@@ -274,20 +246,25 @@ func amqpDeliveries(ctx context.Context, opts Opts, lockPool *pgxpool.Pool, stor
 
 	ds := make([]*notifier.Delivery, 0, deliveries)
 	for i := 0; i < deliveries; i++ {
-		distLock := pgdl.NewPool(lockPool, 0)
+		// Can't share a ctxlock because the Deliverer object unconditionally
+		// spawns background goroutines.
+		l, err := ctxlock.New(ctx, lockPool)
+		if err != nil {
+			return fmt.Errorf("failed to create locker: %w", err)
+		}
 		if conf.Direct {
 			q, err := namqp.NewDirectDeliverer(conf)
 			if err != nil {
 				return fmt.Errorf("failed to create AMQP deliverer: %v", err)
 			}
-			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, distLock)
+			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, l)
 			ds = append(ds, delivery)
 		} else {
 			q, err := namqp.New(conf)
 			if err != nil {
 				return fmt.Errorf("failed to create AMQP deliverer: %v", err)
 			}
-			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, distLock)
+			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, l)
 			ds = append(ds, delivery)
 		}
 	}
@@ -316,20 +293,25 @@ func stompDeliveries(ctx context.Context, opts Opts, lockPool *pgxpool.Pool, sto
 
 	ds := make([]*notifier.Delivery, 0, deliveries)
 	for i := 0; i < deliveries; i++ {
-		distLock := pgdl.NewPool(lockPool, 0)
+		// Can't share a ctxlock because the Deliverer object unconditionally
+		// spawns background goroutines.
+		l, err := ctxlock.New(ctx, lockPool)
+		if err != nil {
+			return fmt.Errorf("failed to create locker: %w", err)
+		}
 		if conf.Direct {
 			q, err := stomp.NewDirectDeliverer(conf)
 			if err != nil {
 				return fmt.Errorf("failed to create STOMP direct deliverer: %v", err)
 			}
-			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, distLock)
+			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, l)
 			ds = append(ds, delivery)
 		} else {
 			q, err := stomp.New(conf)
 			if err != nil {
 				return fmt.Errorf("failed to create STOMP deliverer: %v", err)
 			}
-			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, distLock)
+			delivery := notifier.NewDelivery(i, q, opts.DeliveryInterval, store, l)
 			ds = append(ds, delivery)
 		}
 	}
